#!/bin/sh
# %W% %G% %U% - %Q% #
# ident "@(#)VCS:%M% %I%"
#
#  $Copyright: Copyright (c) 2011 Symantec Corporation.
#  All rights reserved.
#
#  THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#  SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#  WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#  The Licensed Software and Documentation are deemed to be commercial
#  computer software as defined in FAR 12.212 and subject to restricted
#  rights as defined in FAR Section 52.227-19 "Commercial Computer
#  Software - Restricted Rights" and DFARS 227.7202, "Rights in
#  Commercial Computer Software or Commercial Computer Software
#  Documentation", as applicable, and any successor regulations. Any use,
#  modification, reproduction release, performance, display or disclosure
#  of the Licensed Software and Documentation by the U.S. Government
#  shall be solely in accordance with the terms of this Agreement.  $

if [ -x "/usr/bin/perl" ]; then
        PERL=/usr/bin/perl
elif [ -x "/opt/VRTSperl/bin/perl" ]; then
        PERL=/opt/VRTSperl/bin/perl
else
        PERL=`/usr/bin/which perl | /usr/bin/grep -v '^no'`
        if [ -z "$PERL" ]; then
                echo "Cannot find a version of perl to execute script"
                exit 1
        fi
fi
eval 'exec $PERL  -x -S $0 ${1+"$@"}'
        if 0;
#!perl

$|=1;

use File::Basename;
require("/opt/VRTSagents/ha/bin/Oracle/OraDiscoveryLib.pm");

my ($PATH) = $ENV{PATH};
$PATH = "/usr/bin:/bin:/usr/sbin:/sbin:$PATH";
$ENV{PATH} = $PATH;

$CAT="cat";
$CUT="cut";
$GREP="grep";
$IFCONFIG="ifconfig";
$SED="sed";
$UNIQ="uniq"; 
$AWK="awk"; 
$BLKID="blkid";

$MOUNTSCRIPT="/opt/VRTSagents/ha/bin/Oracle/OraMountDiscovery.pl" ;

@INSTANCES;
@LISTENERS;
@DISCINFO;
@MPNTS;
@ADDRESSES;
$HOME;
$MOUNTSCRIPT;
$hostname;

sub discover_instances
{
        $MOUNTSCRIPT="/opt/VRTSagents/ha/bin/Oracle/OraMountDiscovery.pl" ;
        open(ORA,"$PS -ef | $GREP ora_pmon | $GREP -v grep |");
        while($line = <ORA>)
        {
                chomp($line);
                $line =~ s/^\s+//;
                @tok=split(/\s+/, $line);
                $num = $#tok;
                @sid=split(/_+/,$tok[$num]);
                $pid = $tok[1];
		$user = getUserName($pid);
                $oracle_home = getOracleHome($pid);
                push(@INSTANCES, "$user $sid[2] $oracle_home");
        }
	foreach $instance(@INSTANCES)
        {
                @tok=split(/\s+/, $instance);
                $owner=$tok[0];
                $sid = $tok[1];
                $home=$tok[2];

                print("STARTINSTANCE\n");
                print "$sid $owner $home ";
                $version = getInstanceVersion($owner, $sid, $home);
                print("$version ");
                $spfile = getSPfile($owner, $sid, $home);
                if($spfile !~ /^\//)
                {
                        $spfile = "";
                }
                print("$spfile\n");
                $ret = isDBMounted($owner, $sid, $home);
                if($ret eq "0") #fatal
                {
                        print("ERROR:NOTMOUNTED\n");
                }
		elsif(isDatabaseSuspended($owner, $sid, $home) eq "TRUE")
		{
			print("ERROR:SUSPENDED\n");
		}
		else
                {
                        $isParallel = isDatabaseParallel($owner, $sid, $home);
                        if($isParallel eq "TRUE")
                        {
                                print("ERROR:PARALLEL\n");
                        }
                        else
                        {
                                print("SUCCESS\n");
                        }
			getMounts($owner, $sid, $home, $version);
                        foreach $mp(@MPNTS)
                        {
                                #print database file name if it starts with a /
                                if($mp =~ /^\//)
                                {
                                        print "$mp\n";
                                }
                                elsif($mp =~ /^ORA-/)
                                {
                                        print "$mp\n";
                                }
                        }
                }
                print("ENDINSTANCE\n");
        }
}

sub discover_listeners
{
        my(%tnshash) = @_;
        @LISTENERS = getListeners();
        foreach $lsnr(@LISTENERS)
        {
                @tok=split(/\s+/, $lsnr);
                $owner=$tok[0];
                $home=$tok[1];
		$lsnrname=$tok[2];
		$tnsadmin = $tnshash{$lsnrname};
		if($tnsadmin eq "")
		{
			$tnsadmin = "$home/network/admin";
		}
                $ret = runListenerScript($owner, $lsnrname, $home, $tnsadmin);
		next if ($ret != 0);
                print("STARTLISTENER\n");
                print("$lsnrname $owner $home $tnsadmin");
                @ADDRESSES = getListenerIP("$tnsadmin/listener.ora", $lsnrname);
                foreach $addr(@ADDRESSES)
                {
                        print(" ");
                        print("$addr");
                }
                print("\n");
                foreach $instance(@DISCINFO)
                {
                        print("$instance\n");
                }
                print("ENDLISTENER\n")
        }
}

sub discover_databases
{
        open(ORA,"$PS -ef | $GREP ora_pmon | $GREP -v grep |");
        while($line = <ORA>)
        {
                chomp($line);
                $line =~ s/^\s+//;
                @tok=split(/\s+/, $line);
                $num = $#tok;
                @sid=split(/_+/,$tok[$num]);
                $pid = $tok[1];
		$user = getUserName($pid);
                $oracle_home = getOracleHome($pid);
                push(@INSTANCES, "$user $sid[2] $oracle_home");
        }
        foreach $instance(@INSTANCES)
	{
		@tok=split(/\s+/, $instance);
		$owner=$tok[0];
		$sid=$tok[1];
		$home=$tok[2];
       		$dbdetails = getDatabaseDetails($owner, $sid, $home, $hostname);
            	if( $dbdetails ne "FALSE")
            	{
			print("STARTDB\n");
               		print("$dbdetails\n");
               		print("ENDDB\n");
            	}
        }
}

sub discover_instance_count
{
        $cnt = getInstanceCount();
        print("$cnt\n");
        exit(0);
}

sub discover_listener_count
{
        $cnt = getListenerCount();
        print("$cnt\n");
        exit(0);
}

sub get_lvmvg_disks
{
    my ($device) = @_;
    my (@tok, $lvg, $disks, $line);
    my ($PVS, $VGDISPLAY) = ("pvs", "vgdisplay");

    @tok = split (/\//, $device);
    ($lvg) = split (/-/, $tok[3]);

    # check if it is LVM2
    return if (open (VGHANDLE, "$VGDISPLAY $lvg 2>/dev/null \| $GREP -i 'Format\.\*lvm2' >/dev/null |") == 0);
    close (VGHANDLE);

    return if (open (PVHANDLE, "$PVS | $AWK -v VG=$lvg '{if (\$2 == VG) print \$1 }' \| $SED -e 's/[0-9]*//g' \| $UNIQ |") == 0);

    while ($line = <PVHANDLE>)
    {
        chomp ($line);
        $disks .= " $line";
    }
    close (PVHANDLE);

    return $disks;
}

sub discover_mount
{
    my (@vfstabs, @mounts, %all_mounts, %share_mounts);
    my ($line, $bd, $mp, $fs, $mo, $tmp, $info, $mp_in_mount, $mp_in_vfstab);
    my ($MOUNT, $VFSTAB, $MNTTAB, @tok);

    $MOUNT="mount";
    $VFSTAB="/etc/fstab";
    $MNTTAB="/etc/mtab";

    # read /etc/vfstab file
    die if (open (FILEHANDLE, "$CAT $VFSTAB |") == 0);
    while ($line = <FILEHANDLE>)
    {
        chomp ($line);
        $line =~ s/\s+/ /g;
        $line =~ s/^\s//g;
        ($bd, $mp, $fs, $tmp) = split (/ /, $line, 4);

        $first_char = substr ($bd, 0, 1);
        next if ($first_char eq "#"); # ignore if commented

        push (@vfstabs, $mp);
    }
    close (FILEHANDLE);

    # read mount command
    die if (open (FILEHANDLE, "$MOUNT |") == 0);
    while ($line = <FILEHANDLE>)
    {
        chomp ($line);
        ($bd, $tmp, $mp, $tmp, $fs, $mo) = split (/\s+/, $line);
        if ($bd =~ /^LABEL=/i)
        {
            $bd =~ s/^LABEL=//i;
            next if (open (BLKIDHANDLE, "$BLKID -t LABEL=$bd -o device |") == 0);	
            while ($blkdev = <BLKIDHANDLE>)
            {
                next if ($blkdev =~ /\/dev\/vx\/rdmp/);
                next if ($blkdev !~ /^\//);
                chomp ($blkdev);
                $bd = $blkdev;
		last;
            }
        }
        next if ($bd !~ /^\//);

        chop ($mo); # remove ')' character
        $mo = substr ($mo, 1); # remove '(' character

        # if blockdevice is volume of rootdg, ignore it
        if ($bd =~ /\/dev\/vx\/dsk\//)
        {
            @tok = split (/\//, $bd);
            next if ($#tok == 4);
            next if ("$tok[4]" eq "rootdg");
        }

        $info = "$bd $mp $fs $mo";
        # if LVM2, add disks

        if ($bd =~ /\/dev\/mapper\//)
        {
            $disks = get_lvmvg_disks ($bd);
            $info = "$info disks $disks";
        }

        $all_mounts{$mp} = $info;
        push (@mounts, $mp);
    }
    close (FILEHANDLE);

L1: foreach $mp_in_mount (@mounts)
    {
        foreach $mp_in_vfstab (@vfstabs)
        {
            #next L1 if ($mp_in_mount eq $mp_in_vfstab);
            next L1 if (! defined $all_mounts{$mp_in_mount});
        }

	print("$all_mounts{$mp_in_mount}\n");
    }

}


sub discover_ip
{
    my ($line, @tok, $ipaddr, $netmask, $device, @parts, $i, $tmp);

    die if (open (NIC, "$IFCONFIG |") == 0);
    while ($line = <NIC>)
    {
	chomp ($line);
        $line =~ s/^\s+//;
        
        if ($line =~ /Link encap/)
        {
            @tok = split (/\s+/, $line);
            $device = $tok[0];
            
            ($device, $tmp) = split (/:/, $device, 2);
        }
        elsif ($line =~ /^inet/ && $device ne "" && $line !~ /inet6/)
        {
            @tok = split (/\s+/, $line);

            for ($i = 0; $i <= $#tok; ++$i)
            {
                if ($tok[$i] =~ /^addr:/i) # get ip address
                {
                    $ipaddr = $tok[$i];
                    ($tmp, $ipaddr) = split (/:/, $ipaddr, 2);
                }
                elsif ($tok[$i] =~ /^Mask:/i) # get netmask
                {
                    $netmask = $tok[$i];
                    ($tmp, $netmask) = split (/:/, $netmask, 2);
                    last;
                }
            }

            print ("$ipaddr $netmask $device\n");

            $device = "";
        }
    }

    exit (0);
}


sub discover_nic
{
    my ($line);

    open (NIC, "$IFCONFIG | $GREP -v lo | $CUT -d' ' -f1 | $GREP -v : |");
    while($line = <NIC>)
    {
	chomp($line);
	next if($line eq "");
        print("$line\n");
    }
    exit (0);
}


sub usage
{
    my ($file);

    $file = basename (__FILE__);

    print "Usage:$file [instances|listeners|databases|ip|nic|mount|lltlinks|crshome]\n";
    exit (1);
}

$hostname = `$HOSTNAME`;
chomp($hostname);

if ($#ARGV ne 0)
{
    usage ();
}
elsif ($ARGV[0] eq "instances")
{
    discover_instances ();
}
elsif ($ARGV[0] =~ /^listeners/)
{
	my %tnshash;
	@args = split(/:/, $ARGV[0]);
	shift(@args);
	foreach $tnsloc(@args)
	{
		($lname, $tnsadmin) = split(/=/, $tnsloc, 2);
		$tnshash{$lname} = $tnsadmin;
	}
    	discover_listeners(%tnshash);
}
elsif ($ARGV[0] eq "databases")
{
    discover_databases ();
}
elsif ($ARGV[0] eq "icnt")
{
    discover_instance_count ();
}
elsif ($ARGV[0] eq "lcnt")
{
    discover_listener_count ();
}
elsif($ARGV[0] eq "nondeftns")
{
	$ret = getLsnrsWithNonDefTnsAdmin();
	print("$ret");
	exit(0);
}
elsif($ARGV[0] eq "lnames")
{
    @LISTENERS = getListeners();
    # Discover details for each listener
    foreach $lsnr(@LISTENERS)
    {
            print("$lsnr\n");
    }
}
elsif ($ARGV[0] eq "ip")
{
    discover_ip ();
}
elsif ($ARGV[0] eq "mount")
{
    discover_mount ();
}
elsif ($ARGV[0] eq "nic")
{
    discover_nic ();
}
elsif ($ARGV[0] eq "lltlinks")
{
    discover_lltlinks();
}	
elsif ($ARGV[0] eq "crshome")
{
    $crs_home = discover_crs_home();
    print("$crs_home\n");
}
else
{
    usage ();
}
