#!/bin/bash
# $Id: ApplicationDiscovery.sh,v 2.6 2006/04/21 13:45:45 gpathare Exp $
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:$PATH

discover_user ()
{
    cat /etc/passwd | \
    grep -v ^"#" |grep -v "/dev/null" | grep -v "/bin/false" | \
    grep -v "/sbin/nologin" | grep -v -w ^shutdown | grep -v -w ^bin | \
    grep -v -w ^daemon | grep -v -w ^adm | grep -v -w ^lp | \
    grep -v -w ^sync | grep -v -w ^halt | grep -v -w ^mail | \
    grep -v -w ^news | grep -v -w ^uucp | grep -v -w ^operator | \
    grep -v -w ^games | grep -v -w ^gopher | grep -v -w ^ftp | \
    grep -v -w ^nobody | grep -v -w ^mailnull | grep -v -w ^rpm | \
    grep -v -w ^xfs | grep -v -w ^wnn | grep -v -w ^ntp | \
    grep -v -w ^rpc | grep -v -w ^gdm | grep -v -w ^rpcuser | \
    grep -v -w ^nfsnobody | grep -v -w ^nscd | grep -v -w ^ident | \
    grep -v -w ^radvd | grep -v -w ^apache | grep -v -w ^squid | \
    grep -v -w ^named | grep -v -w ^pcap | grep -v -w ^pvm | \
    grep -v -w ^piranha | grep -v -w ^netdump | grep -v -w ^amanda | \
    grep -v -w ^junkbust | grep -v -w ^mailman | grep -v -w ^mysql | \
    cut -f1 -d":"
}


discover_mount ()
{
    local mounts=""
    local fstabs=""
    local shared=""
    local bd=""
    local mp=""
    local fs=""
    local mo=""
    local vg=""
    local first_char=""
    local tmp=""
    local lvdisks=""

    fstabs=`cat /etc/fstab | tr -s " " " " | sed -e 's/^ //g' | \
            grep -v ^"#" | cut -f2 -d" "`

    mounts=`mount | grep -w -e "ext2" -e "ext3" -e "vxfs" -e "reiserfs" \
             | tr -s " " " " | sed -e 's/^ //g' | cut -f1,3,5,6 -d" "`

    for i in ${mounts}; do

        if [ "${bd}" = "" ]; then
            bd="${i}"
            continue
        elif [ "${mp}" = "" ]; then
            mp="${i}"
            continue
        elif [ "${fs}" = "" ]; then
            fs="${i}"
            continue
        elif [ "${mo}" = "" ]; then
            mo="${i}"

            first_char=`echo ${bd} | cut -b1`

            if [ "${fs}" != "nfs" ] && [ "${first_char}" != "/" ]; then
                bd=""
                mp=""
                fs=""
                mo=""
                continue
            fi
        fi

        for j in ${fstabs}; do
            if [ "${j}" = "${mp}" ]; then
                bd=""
                mp=""
                fs=""
                mo=""
                continue 2
            fi
        done

        # if the blockdevice is volume of rootdg, ignore it
        tmp=`echo ${bd} | grep ^"/dev/vx/dsk/"`
        if [ "${tmp}" ]; then
            tmp=`echo ${bd} | cut -f6 -d"/"`
            if [ -z "${tmp}" ]; then
                bd=""
                mp=""
                fs=""
                mo=""
                continue
            fi

            tmp=`echo ${bd} | cut -f5 -d"/"`
            if [ "${tmp}" = "rootdg" ]; then
                bd=""
                mp=""
                fs=""
                mo=""
                continue
            fi
        fi

        if [ "${shared}" = "" ]; then
            shared="${bd} ${mp} ${fs} ${mo}"
        else
            shared="${shared}\n${bd} ${mp} ${fs} ${mo}"
        fi

        # if the blockdevice is logical volume
        tmp=`echo ${bd} | grep ^"/dev/mapper/"`
        if [ -z "${tmp}" ]; then
            bd=""
            mp=""
            fs=""
            mo=""
            continue
        fi

        # get the logical volume group
        vg=`echo ${bd} | cut -f4 -d"/" | cut -f1 -d"-"`

        bd=""
        mp=""
        fs=""
        mo=""

        if [ -z "${vg}" ]; then
            continue
        fi

        # if the LVG is not LVM2, do not proceed
        vgdisplay ${vg} 2>/dev/null | grep -i 'Format.*lvm2' >/dev/null
        if [ $? -ne 0 ]; then
            continue
        fi

        # scan the volume group's disks
        lvdisks=`pvs | awk -v VG=${vg} '{if ($2 == VG) print $1 }' | \
                 sed -e 's/[0-9]*//g' | uniq`

        if [ "${lvdisks}" ]; then
            shared="${shared} disks"
            for j in ${lvdisks}; do shared="${shared} $j"; done
        fi
    done
    
    echo -e "${shared}"
}


discover_nic ()
{
    local cards=""
    cards=`ifconfig | grep -v lo | cut -f1 -d" " | grep -v ":"`
    echo -e "${cards}"
}


discover_ip ()
{
    local vcards=""
    local card=""
    local ip=""
    local mask=""
    RETSTR=""
    
    vcards=`ifconfig | cut -f1 -d" " | grep ":"`
    if [ -z "${vcards}" ]; then
        echo "${RETSTR}"
        return 0
    fi

    for card in ${vcards}; do
        ip=`ifconfig ${card} | grep inet | cut -f2 -d":" | cut -f1 -d" "`
        mask=`ifconfig ${card} | grep inet | cut -f4 -d":" | cut -f1 -d" "`
        card=`echo ${card} | cut -f1 -d":"`

        if [ ! -z "${ip}" ]; then
            if [ -z "${RETSTR}" ]; then
                RETSTR="${ip} ${mask} ${card}"
            else
                RETSTR="${RETSTR}\n${ip} ${mask} ${card}"
            fi
        fi
    done
    
    echo -e "${RETSTR}"
}


usage ()
{
    echo "Usage: `basename $0` [user|mount|nic|ip]"
    exit 1
}


if [ $# -ne 1 ]; then
    usage
fi

arg=$1

if [ "${arg}" = "user" ]; then
    discover_user
elif [ "${arg}" = "mount" ]; then
    discover_mount
elif [ "${arg}" = "nic" ]; then
    discover_nic
elif [ "${arg}" = "ip" ]; then
    discover_ip
else
    usage
fi

exit 0
