#!/opt/VRTSperl/bin/perl -I../../lib
#
#$Copyright: Copyright (c) 2011 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
#

#
# Library of functions for the NFSRestartRes Agent.
#
package LockAgentLib;
use Exporter;
@ISA = ('Exporter');
@EXPORT = ('getBasename', 'getProcPid', 'getProcOptions', 'AcquireLock', 'ReleaseLock', 'Killlockingdaemons', 'CheckProc', 'KillProc', 'StartProc', 'KillNFSdaemons');

my ($LOCK_SH, $LOCK_EX, $LOCK_UN, $vcs_home);
my ($startsrc, $stopsrc) = ("/usr/bin/startsrc", "/usr/bin/stopsrc");
$kill="/bin/kill";
#
# Initialize variables.
#
$LOCK_SH = 1;
$LOCK_EX = 2;
$LOCK_UN = 8;

#
# Acquires an exclusive lock on lockfh.
#
sub AcquireLock {
    my($lockfh) = @_;
    my($err);

    $err = flock($lockfh, $LOCK_EX);
    return $err;
}

#
# Releases the exclusive lock.
#
sub ReleaseLock {
    my($lockfh) = @_;

    flock($lockfh, $LOCK_UN);
    return;
}

#
# getBasename ()
#
# Didn't want to call it basename() since there already is a 
# library module with that name.  Since the 'strict' module wasn't 
# available with the VxCS Perl I'm assuming that the File::Basename 
# module isn't either.
#
sub getBasename {
    my ($i, $cmd);

    $cmd=$_[0];
    $i = rindex ($cmd, "/");
    if ( $i != -1) {
        $cmd = substr $cmd, $i+1;
    }
    chomp ($cmd);
    return ($cmd);
}

#
# getProcPid ()
#
# Didn't want to call it getpid() because perl might have a
# getpid() function one day (it has just about everything else).
#
sub getProcPid {
    my ($target, $pid);

    $target=$_[0];
    $pid = "";
    open (PIDS, "ps -ea -o comm,pid --no-header 2> /dev/null |");
    while (<PIDS>) {
        @tmp = split;
        $cmdname = getBasename ($tmp[0]);
        if ($cmdname eq $target) {
            if (length $pid) {
                $pid = sprintf ("%s %s", $pid, $tmp[1]);
            }
            else {
                $pid=$tmp[1];
            }
        }
    }
    return ($pid);
}

#
# getProcOptions ()
#       Get options associated with the command
#
sub getProcOptions {
        my ($target, $pid, $proc, @proc_options, $junk);
        $target=$_[0];
        open (PIDS, "ps -ea -o comm,args --no-header 2> /dev/null |");
        while (<PIDS>) {
                @tmp = split;
                $cmd = getBasename ($tmp[0]);
                if ($cmd eq $target) {
                        ($junk, $junk, @proc_options) = @tmp;
                        $proc=join(" ", @proc_options);
                        last;
                }
        }
        return ($proc);
}

#
# kill statd and lockd
#
sub Killlockingdaemons {
    if( -e "/etc/redhat-release")
    {
	    system("service nfslock stop > /dev/null");
#        KillProc("rpc.statd","");
    }
    else
    {
        KillProc("rpc.statd","");
        KillProc("lockd","");
        KillProc("sm-notify","");
    }
    return;
}

sub KillNFSdaemons {
	KillProc("nfsd", "");
	KillProc("rpc.mountd","");
	if ( -e "/etc/SuSE-release") {
		system("/etc/init.d/nfsserver stop");
	}
}
#
# Start a process which is running $target (without the path).
# and has an argument which is an exact match of $arg.
#
sub CheckProc {
my($target, $arg) = @_;
my ($pid, $proc, @proc_options, $junk, $retval);
$retval = "";

$retval = "";
open (PIDS, "ps -ea -o comm,pid,args --no-header 2> /dev/null |");
while (<PIDS>) {
@tmp = split;
$cmd = getBasename ($tmp[0]);
		if ($cmd eq $target) {
			$proc = "";
			($junk, $pid, @proc_options) = @tmp;
			$proc=join(" ", @proc_options);
                        if ($proc =~ /\b$arg\b/) {
				$retval = "$pid $retval";
                        }
		}
	}
	return $retval;
}

#
# Kill a process which is running $target (without the path).
# and has an argument which is an exact match of $arg.
#
sub KillProc {
    my($target, $arg) = @_;
    my($pid, @pidlist);

    $pid = CheckProc($target, $arg);
    if ($pid) {
        @pidlist = split(/ /,$pid);
        kill 15, @pidlist if ($#pidlist != -1);
        kill 9,  @pidlist if ($#pidlist != -1);
    }
    return $pid;
}

#
# fork a process to start $cmd (with full path)
# running with arguments $arg.
#
# On success it returns the pid.
# on failure it returns -1, with
# $! containing the errno.
#
sub StartProc {
    my($cmd, $arg) = @_;
    my($pid);

    if (-x $cmd) {
        if ($pid = fork) {
        #
        # This is the parent.
        #
            return $pid;
        } elsif ($pid == 0) {
        #
        # Child.
        #
            exec("$cmd $arg");
            exit(1);
        } else {
            return -1;
        }
    }
    $ERRNO = ENOENT;
    return -1;
}

#
#  Return a non-zero to make perl happy.
#
