#!/usr/bin/perl
# $Id: ShareDiscovery.pl,v 2.8 2006/03/26 14:45:55 gpathare Exp $
# $Copyrights: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

use File::Basename;

my ($PATH) = $ENV{PATH};
$PATH = "/usr/bin:/bin:/usr/sbin:/sbin:$PATH";
$ENV{PATH} = $PATH;

$AWK="awk";
$CAT="cat";
$CUT="cut";
$GREP="grep";
$IFCONFIG="ifconfig";
$PS="ps";
$SED="sed";
$UNIQ="uniq";
$WC="wc";

$LOCK_DIR = "/var/lib/nfs";


sub get_lvmvg_disks
{
    my ($device) = @_;
    my (@tok, $lvg, $disks, $line);
    my ($PVS, $VGDISPLAY) = ("pvs", "vgdisplay");

    @tok = split (/\//, $device);
    ($lvg) = split (/-/, $tok[3]);

    # check if it is LVM2
    return if (open (VGHANDLE, "$VGDISPLAY $lvg 2>/dev/null \| $GREP -i 'Format\.\*lvm2' >/dev/null |") == 0);
    close (VGHANDLE);

    return if (open (PVHANDLE, "$PVS | $AWK -v VG=$lvg '{if (\$2 == VG) print \$1 }' \| $SED -e 's/[0-9]*//g' \| $UNIQ |") == 0);

    while ($line = <PVHANDLE>)
    {
        chomp ($line);
        $disks .= " $line";
    }
    close (PVHANDLE);

    return $disks;
}

sub discover_mount
{
    my (@vfstabs, @mounts, %all_mounts, %share_mounts);
    my ($line, $bd, $mp, $fs, $mo, $tmp, $info, $mp_in_mount, $mp_in_vfstab);
    my ($lvg, $lvol, $disks);
    my ($MOUNT, $VFSTAB, $MNTTAB, @tok);

    $MOUNT="/bin/mount";
    $VFSTAB="/etc/fstab";
    $MNTTAB="/etc/mtab";

    # read /etc/vfstab file
    die if (open (FILEHANDLE, "$CAT $VFSTAB |") == 0);
    while ($line = <FILEHANDLE>)
    {
        chomp ($line);
        $line =~ s/\s+/ /g;
        $line =~ s/^\s//g;
        ($bd, $mp, $fs, $tmp) = split (/ /, $line, 4);

        $first_char = substr ($bd, 0, 1);
        next if ($first_char eq "#"); # ignore if commented
    push (@vfstabs, $mp);
    }
    close (FILEHANDLE);

    # read mount command
    die if (open (FILEHANDLE, "$MOUNT |") == 0);
    while ($line = <FILEHANDLE>)
    {
        chomp ($line);
        next if ($line !~ /^\//);
        ($bd, $tmp, $mp, $tmp, $fs, $mo) = split (/\s+/, $line);
        chop ($mo); # remove ')' character
        $mo = substr ($mo, 1); # remove '(' character
	 # if blockdevice is volume of rootdg, ignore it
        if ($bd =~ /\/dev\/vx\/dsk\//)
        {
            @tok = split (/\//, $bd);
            next if ($#tok == 4);
            next if ("$tok[4]" eq "rootdg");
        }

        $info = "$bd $mp $fs $mo";

        # if LVM2, add disks
        if ($bd =~ /\/dev\/mapper\//)
        {
            $disks = get_lvmvg_disks ($bd);
            $info = "$info disks $disks";
        }

        $all_mounts{$mp} = $info;
        push (@mounts, $mp);
    }
    close (FILEHANDLE);
L1: foreach $mp_in_mount (@mounts)
    {
        foreach $mp_in_vfstab (@vfstabs)
        {
            next L1 if ($mp_in_mount eq $mp_in_vfstab);
            next L1 if (! defined $all_mounts{$mp_in_mount});
        }

        $share_mounts{$mp_in_mount} = $all_mounts{$mp_in_mount};
    }

    return %share_mounts;
}

sub discover_nfs
{
    my ($nproc, $line, %mounts);

    open (NFS, "$PS -eo args | $GREP -v grep | $GREP -w nfsd | $WC -l |");
    while($nproc = <NFS>)
    {
        chomp ($nproc);
        last;
    }
    close (NFS);

    %mounts = discover_mount ();
    $line = $mounts{$LOCK_DIR};

    if (defined ($line))
    {
        print "$nproc 1 $line\n";
    }
    else
    {
        print "$nproc 0\n";
    }
}

sub discover_share
{
    my ($line, @tok, $dir, $options, %mounts, $mp, $bdir, $client);

    $SHARE="$LOCK_DIR/etab";

    %mounts = discover_mount ();

    #foreach $mp (sort keys %mounts)
    #{
    #    print "$mp = $mounts{$mp}\n";
    #}

    #exit (0);

    die if (open (FILEHANDLE, "$CAT $SHARE |") == 0);
L2: while ($line = <FILEHANDLE>)
    {
	chomp ($line);
        $line =~ s/\s+/ /g;
        $line =~ s/^\s//;

        @tok = split (/ /, $line);
        $dir = $tok[0];
        $options = $tok[1];

        ($client, $options) = split (/\(/, $options);

        $client = "*" if ($client eq "");
        chop ($options); # remove ')' character

        # check if the directory exists
        next unless (-d "$dir");

        $bdir = $dir;

        while ($bdir ne "/")
        {
            if (defined ($mounts{$bdir}))
            {
                print "$dir $client $options $mounts{$bdir}\n";
                next L2;
            }
            else
            {
                $bdir = dirname ($bdir);
            }
        }
    }
    close (FILEHANDLE);
    exit (0);
}


sub discover_ip
{
    my ($line, @tok, $ipaddr, $netmask, $device, @parts, $i, $tmp);

    die if (open (NIC, "$IFCONFIG |") == 0);
    while ($line = <NIC>)
    {
        chomp ($line);
        $line =~ s/^\s+//;
        
        if ($line =~ /^eth\d+:\d+\s+/)
        {
            @tok = split (/\s+/, $line);
            $device = $tok[0];
            
            ($device, $tmp) = split (/:/, $device, 2);
        }
        elsif ($line =~ /^inet/ && $device ne "")
        {
            @tok = split (/\s+/, $line);

            for ($i = 0; $i <= $#tok; ++$i)
            {
                if ($tok[$i] =~ /^addr:/i) # get ip address
                {
                    $ipaddr = $tok[$i];
                    ($tmp, $ipaddr) = split (/:/, $ipaddr, 2);
                }
                elsif ($tok[$i] =~ /^Mask:/i) # get netmask
                {
                    $netmask = $tok[$i];
                    ($tmp, $netmask) = split (/:/, $netmask, 2);
                    last;
                }
            }

            print ("$ipaddr $netmask $device\n");

            $device = "";
        }
    }

    exit (0);
}


sub discover_nic
{
    my ($line);

    open (NIC, "$IFCONFIG | $GREP -v lo | $CUT -d' ' -f1 | $GREP -v : |");
    while($line = <NIC>)
    {
        print("$line");
    }
    exit (0);
}

sub discover_mount_only
{
    my (@vfstabs, @mounts, %all_mounts, %share_mounts);
    my ($line, $bd, $mp, $fs, $mo, $tmp, $info, $mp_in_mount, $mp_in_vfstab);
    my ($lvg, $lvol, $disks);
    my ($MOUNT, $VFSTAB, $MNTTAB, @tok);

    $MOUNT="/bin/mount";
    $VFSTAB="/etc/fstab";
    $MNTTAB="/etc/mtab";

    # read mount command
    die if (open (FILEHANDLE, "$MOUNT |") == 0);
    while ($line = <FILEHANDLE>)
    {
        chomp ($line);
        next if ($line !~ /^\//);
        ($bd, $tmp, $mp, $tmp, $fs, $mo) = split (/\s+/, $line);
        chop ($mo); # remove ')' character
        $mo = substr ($mo, 1); # remove '(' character

        # if blockdevice is volume of rootdg, ignore it
        if ($bd =~ /\/dev\/vx\/dsk\//)
        {
            @tok = split (/\//, $bd);
            next if ($#tok == 4);
            next if ("$tok[4]" eq "rootdg");
        }

        $info = "$bd $mp $fs $mo";

        # if LVM2, add disks
        if ($bd =~ /\/dev\/mapper\//)
        {
            $disks = get_lvmvg_disks ($bd);
            $info = "$info disks $disks";
        }
	 $all_mounts{$mp} = $info;
        push (@mounts, $mp);
    }
    close (FILEHANDLE);
L1: foreach $mp_in_mount (@mounts)
    {
        print "$all_mounts{$mp_in_mount}\n";
        next L1;
    }
    exit(0);
}

sub get_mount_fsid0
{
 my ($line, @tok, @tok1, $dir, $options, %mounts, $mp, $bdir, $client, $indv_opt);

    $SHARE="$LOCK_DIR/etab";

    %mounts = discover_mount ();

    die if (open (FILEHANDLE, "$CAT $SHARE |") == 0);
L2: while ($line = <FILEHANDLE>)
    {
        chomp ($line);
        $line =~ s/\s+/ /g;
        $line =~ s/^\s//;

        @tok = split (/ /, $line);
        $dir = $tok[0];
        $options = $tok[1];

        ($client, $options) = split (/\(/, $options);

        $client = "*" if ($client eq "");
        chop ($options); # remove ')' character

        # check if the directory exists
        next unless (-d "$dir");

        $bdir = $dir;

        while ($bdir ne "/")
        {
            if (defined ($mounts{$bdir}))
            {

		@tok1 = split(/\,/, $options);
		foreach $indv_opt (@tok1)
		{
			if($indv_opt eq "fsid=0")
			{
				print "$dir $client $options $mounts{$bdir}\n";
				next L2;
			}
		}
		next L2;			
            }
            else
            {
                $bdir = dirname ($bdir);
            }
        }
    }
    close (FILEHANDLE);
    exit (0);
}


sub usage
{
    my ($file);

    $file = basename (__FILE__);

    print "Usage:$file [ip|nfs|nic|share|mount|fsid]\n";
    exit (1);
}



if ($#ARGV ne 0)
{
    usage ();
}
elsif ($ARGV[0] eq "share")
{
    discover_share ();
}
elsif ($ARGV[0] eq "ip")
{
    discover_ip ();
}
elsif ($ARGV[0] eq "nfs")
{
    discover_nfs ();
}
elsif ($ARGV[0] eq "nic")
{
    discover_nic ();
}
elsif ($ARGV[0] eq "mount")
{
    discover_mount_only ();
}
elsif ($ARGV[0] eq "fsid")
{
    get_mount_fsid0();
}
else
{
    usage ();
}
