# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

#!/bin/sh

# Commands used
ESXCFG_VSWITCH="/usr/sbin/esxcfg-vswitch"
ESXCFG_VSWIF="/usr/sbin/esxcfg-vswif"
SSH="/usr/bin/ssh"
GREP="/bin/grep"
IFCONFIG="/sbin/ifconfig"
CUT="/bin/cut"
WC="/usr/bin/wc"

# Local definitions
TRUE=1
FALSE=0
SUCCESS=0
ERROR=1
flag_for_lltcount=$TRUE
flag_new_switch=$FALSE
lltcount=1
i=1
flag_for_use_defaults=$FALSE

#Local Functions
usage() 
{
    echo "Usage Error:"
    echo "./configure_vnics.sh <no arguments> - User input required"
    echo "./configure_vnics.sh -s <List of vmnics in order and separated by spaces>"
}

information()
{
    echo ""
    echo "Use this script to set up virtual switches that you want mapped to specific physical NICs and to configure virtual interfaces for portgroups on virtual switches."
    echo ""
    echo "You need two virtual interfaces that are used as private links to correctly configure LLT on ESX Server nodes. To create a virtual interface, you must map it to a portgroup. The portgroup, in turn, is configured on a virtual switch."
    echo ""
    echo "If you know the physical NIC that you want to use, this script can recognize the virtual switch that is mapped to the physical NIC. If no virtual switch is mapped to the physical NIC, use this script to create the virtual switches, portgroups, and virtual interfaces for you."
    echo ""
}

exit_with_error()
{
    echo "ERROR in execution of this script."
    exit $ERROR
}

exit_with_success()
{
    echo "SUCCESS in execution of this script."
    exit $SUCCESS
}

#Check if Silent option was passed
if [ "$#" -eq "1" ]; then
    usage
    exit_with_error
fi

if [ "$#" -ge "2" ]; then
    if [ "$1" = "-s" ]; then
        flag_for_use_defaults=$TRUE
        shift
    else
        # More options to be handled here
        usage
        exit_with_error
    fi
fi

# Number of times to loop
while_loop_count="$#"

# Used to generate the IP for the Virtual Interface created
ret=`$IFCONFIG vswif0 | grep inet`
linkid=`echo $ret | $CUT -d " " -f 2 | $CUT -d ":" -f 2 | $CUT -d "." -f 4` 

#Information text to print
if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    information
fi 

#WHILE LOOP STARTS HERE
while [ "$i" == "1" ]; do

#Reading VMNIC information from arguments or user
if [ "$flag_for_use_defaults" -eq "$TRUE" ]; then
    VMNIC=$1
    shift
    if [ "$VMNIC" == "" ]; then
        usage
        exit_with_error
    fi
fi

if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    tmp=`$IFCONFIG -a | $GREP vmnic | $CUT -d " " -f 1 | $CUT -d "0" -f 2`
    echo "List of available vmnics to use are: $tmp"
    echo "Enter the name of the physical NIC that you want to use."
    read VMNIC
fi

tmp=`echo "$VMNIC" | grep "vmnic"`
if [ "$tmp" == "" ]; then
    tmp=`$IFCONFIG -a | $GREP vmnic | $CUT -d " " -f 1 | $CUT -d "0" -f 2`
    echo "List of available vmnics to use are: $tmp"
    echo "Specify a valid physical NIC name."
    exit_with_error
fi

# Check if this Physical NIC is already mapped to a Virtual Switch
ret=`$ESXCFG_VSWITCH -l | $GREP $VMNIC | cut -d " " -f 1`
if [ "$ret" == "" ]; then
    # No virtual switch is mapped to the physical NIC provided
    flag_new_switch=1
else
    VSWITCH="$ret"
    flag_new_switch=$FALSE
fi

# Create a new virtual switch if necessary
if [ "$flag_new_switch" == "1" ]; then
    VSWITCH="VCS_PVT_$VMNIC"
    if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
        echo "Enter the name of the new virtual switch that would be mapped to $VMNIC. ($VSWITCH)"
        read VSWITCH
        if [ "$VSWITCH" == "" ]; then
            VSWITCH="VCS_PVT_$VMNIC"
        fi
    fi
    ret=`$ESXCFG_VSWITCH -c $VSWITCH`
    if [ "$ret" == "1" ]; then
        if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
            echo "Virtual switch $VSWITCH already exists."
        fi
    else
        if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
            echo "Creating Virtual Switch $VSWITCH..."
        fi
        ret=`$ESXCFG_VSWITCH -a $VSWITCH 2>&1`
        if [ "$?" != "0" ]; then
            echo "Error in creating virtual switch."
            echo "ERROR: $ret"
            exit_with_error
        fi
        if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
            echo "Virtual switch $VSWITCH successfully created."
        fi
    fi

    # At this time, we have a valid VSWITCH name
    # This VSWITCH needs to be mapped to the VMNIC
    if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
        echo "Mapping virtual switch $VSWITCH to physical NIC $VMNIC..."
    fi
    ret=`$ESXCFG_VSWITCH -L $VMNIC $VSWITCH`
    if [ "$?" != "0" ]; then
        echo "Error in mapping virtual switch $VSWITCH to physical NIC $VMNIC."
        echo "$ESXCFG_VSWITCH command returned with error: $ret"
        exit_with_error
    fi
fi 

# IMPORTANT NOTE: 
# If default $PORTGROUP is already exists, there is no command with which 
# we can check if the existing Portgroup belongs to the $VSWITCH above. 
# Hence, this script assumes the uniqueness of the default $PORTGROUP name.
# As a result, this script assumes that if such a portgroup already exists, then
# it does belong to the specified $VSWITCH.
# If not, the user needs to delete the existing $PORTGROUP manually.
# Otherwise the $PORTGROUP would be created.

PORTGROUP="LLT_$VMNIC"

ret=`$ESXCFG_VSWITCH -C $PORTGROUP`
if [ "$ret" == "0" ]; then
    # Create the PORTGROUP 
    ret=`$ESXCFG_VSWITCH -A $PORTGROUP $VSWITCH 2>&1`
    if [ "$?" != "0" ]; then
        echo "Error in configuring portgroup $PORTGROUP on virtual switch $VSWITCH."
        echo "ERROR: $ret"
        exit_with_error
    fi
    if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
        echo "Successfully configured portgroup $PORTGROUP on virtual switch $VSWITCH"
    fi
fi

# At this time $PORTGROUP is configured on the $VSWITCH
vswif_id=`$ESXCFG_VSWIF -l | $WC -l`
vswif_id=`expr $vswif_id - "1"` 
vswif=vswif$vswif_id

if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    echo "Enter the name of the virtual interface: $vswif"
    read VSWIF
    if [ "$VSWIF" == "" ]; then
        VSWIF=$vswif
    fi
else
    VSWIF=$vswif
fi

# Now we will create Virtual Interface VSWIF mapped to PORTGROUP 
ip="192.168.$vswif_id.$linkid"
netmask="255.255.255.0"

if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    echo "Enter the IP address for the virtual interface $VSWIF: $ip"
    read IP
    if [ "$IP" == "" ]; then
        IP="$ip"
    fi
else
    IP=$ip
fi

if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    echo "Enter the Netmask for the virtual interface $VSWIF: $netmask"
    read NETMASK
    if [ "$NETMASK" == "" ]; then
        NETMASK="$netmask"
    fi
else
    NETMASK=$netmask
fi

ret=`$ESXCFG_VSWIF -a $VSWIF -p $PORTGROUP -i $IP -n $NETMASK 2>&1`
if [ "$?" != "0" ]; then
    echo "Error in creating Virtual Interface $VSWIF for Portgroup $PORTGROUP"
    echo "ERROR: $ret"
    exit_with_error
fi

if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    echo "Successfully configured Virtual Interface $VSWIF for Portgroup $PORTGROUP"
else
    echo "$VSWIF "
fi

if [ "$flag_for_use_defaults" -eq "$FALSE" ]; then
    # Increase the lltcount and loop from the beginning
    echo "Do you want to configure another Virtual Interface? [y/n]"
    read ANS

    if [ "$ANS" != "y" ]; then
        exit $SUCCESS
    fi
else
    # This is the case when -s option is passed
    # while_loop_count would be atleast 1 or more

    while_loop_count=`expr $while_loop_count - "1"`
    if [ "$while_loop_count" -eq "0" ]; then
        exit $SUCCESS
    fi
fi

lltcount=`expr $lltcount + $i`

#WHILE LOOP ENDS HERE
done

exit $SUCCESS
