/* 
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $ 
 */

#ifndef _VCSAgApiDefs_h_
#define _VCSAgApiDefs_h_

static const char vcsagapidefs_cvsid[] = "$RCSfile: VCSAgApiDefs.h,v $ $Revision: 1.10.2.5 $";

#ifdef	WINNT
#ifdef	UNICODE
#include <wchar.h>
#endif	/* WINNT && UNICODE */
#endif	/* WINNT */
#include <sys/types.h>
#include "VCSLogTags.h"
#ifdef	LINUX
#include <sys/stat.h>
#endif	/* LINUX */


//
// Contains type definitions used by both the framework & the agent developer.
//

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */

const int VCSAgErrBase = 0;
const int VCSAgErrWouldBlock =	VCSAgErrBase + 1;

const int VCSAgSuccess = 0;
const int VCSAgFailure = 1;

//
// Flags for setting process matching criteria
//
#define PR_UID_SET		0x00000001
#define PR_CONTAINERID_SET	0x00000002

//
// Entry points supported by the agent framework.
//
enum VCSAgEntryPoint {
	VCSAgEPAction,
	VCSAgEPAttrChanged,
	VCSAgEPClean,
	VCSAgEPClose,
	VCSAgEPInfo,
	VCSAgEPMonitor,
	VCSAgEPOffline,
	VCSAgEPOnline,
	VCSAgEPOpen,
	VCSAgEPStartup,
	VCSAgEPShutdown,
	VCSAgEPAMFInit,
	VCSAgEPAMFGetNotification,
	VCSAgEPAMFRegister
};

enum VCSAgBlockingType {
	VCSAgBlocking,
	VCSAgNonblocking
};

enum VCSAgBool {
	VCSAgFalse,
	VCSAgTrue
};

enum VCSAgLockType {
	VCSAgExclusiveLock,
	VCSAgSharedLock
};

//
// Values for the state of a resource.
//
enum VCSAgResState {
	VCSAgResOffline,
	VCSAgResOnline,
	VCSAgResUnknown,
	VCSAgResOfflineFailed,	// monitor() must not return this state.
	VCSAgResOnlineFailed,	// monitor() must not return this state.
	VCSAgResIntentionalOffline // Return code from monitor when it
				// detects the resource has been brought 
				// intentionally offline - available
				// only in V51 and later agents
};

//
// Values for the reason why the clean() entry point is called.
//
enum VCSAgWhyClean {
	VCSAgCleanInit = -1,
	VCSAgCleanOfflineHung,
	VCSAgCleanOfflineIneffective,
	VCSAgCleanOnlineHung,
	VCSAgCleanOnlineIneffective,
	VCSAgCleanUnexpectedOffline,
	VCSAgCleanMonitorHung
};

//
// Values for agent framework version/functionality requested
// via agent registration
//
enum VCSAgAgentVersion {
	V35,
	V40,
	V50,
	V50AMF,
	V51 // New version to support intentional offline changes
};

//
// Values for agent framework version
//
enum VCSAgFwVersion {
	FWV40,
	FWV50
};

//
// Indicates to the Info EP if its the first invocation or
// not. If it is the first call to the info EP, the entry
// point will add the optional fields to the ResourceInfo
// attribute i.e. add and initialize the static and dynamic 
// information in the ResourceInfo attribute and if its not
// the first call, then update the optional fields.
//
enum VCSAgResInfoOp {
	VCSAgResInfoAdd = 1,
	VCSAgResInfoUpdate
};

//
// Switch to specify whether to start or stop amf monitoring
//
enum VCSAgAMFMonitorSwitch {
    VCSAgAMFMonitorStop,
    VCSAgAMFMonitorStart
};

//
// Specifies online or offline (amf) monitoring for the resource
//
enum VCSAgAMFResState {
    VCSAgAMFResOffline,
    VCSAgAMFResOnline
};

//
// Entrypoint struct used by pre-4.0 agents
//
typedef struct {
	void (*open) (const char *res_name, void **attr_val); 
	void (*close) (const char *res_name, void **attr_val);
	VCSAgResState (*monitor) (const char *res_name, void **attr_val, 
		int *conf_level);
	unsigned int (*online) (const char *res_name, void **attr_val); 
	unsigned int (*offline) (const char *res_name, void **attr_val); 
	void (*attr_changed) (const char *res_name, 
		const char *changed_res_name, const char *changed_attr_name,
		void **new_val); 
	unsigned int (*clean) (const char *res_name, VCSAgWhyClean reason, 
		void **attr_val);
	void (*shutdown) ();
} VCSAgEntryPointStruct;

//
// Entrypoint struct to be used by 4.0 agents. Provides two
// new entrypoints - action and info.
//
typedef struct {
	void (*open) (const char *res_name, void **attr_val); 
	void (*close) (const char *res_name, void **attr_val);
	unsigned int (*action) (const char *res_name, const char *action_token,
		void **attr_val, char **action_args, char *action_output);
	VCSAgResState (*monitor) (const char *res_name, void **attr_val, 
		int *conf_level);
	unsigned int (*online) (const char *res_name, void **attr_val); 
	unsigned int (*offline) (const char *res_name, void **attr_val); 
	unsigned int (*info) (const char *res_name, VCSAgResInfoOp resinfo_op,
		void **attr_val, char **info_output, char ***opt_update_args,
		char ***opt_add_args);
	void (*attr_changed) (const char *res_name, 
		const char *changed_res_name, const char *changed_attr_name,
		void **new_val); 
	unsigned int (*clean) (const char *res_name, VCSAgWhyClean reason, 
		void **attr_val);
	void (*shutdown) ();
} VCSAgV40EntryPointStruct;

#ifndef WINNT

// Data structures for procfs scanning in the agent framework
  
// NOTE: These data structures are for agent framework internal use only and
// are subject to change at any time without prior notice. Backward
// compatibility for these data structures is not guaranteed and hence agents
// using them may fail to run on future releases.

// Structure that agent entry points will use to register a process
// for the agent framework to monitor
typedef struct {
	const char *proc_name;
	const char *proc_args;
	int ignore_args;
} VCSAgProcInfoStruct;

// Structure that agent entry points will use to specify the match
// criteria for processes registered for a resource that the agent framework
// will monitor
typedef struct {
	int flags;
	uid_t uid;
	int container_id;
} VCSAgProcMatchCriteriaStruct;

#ifdef	LINUX

// This structure is needed only for Linux because there is no process
// status structure on Linux that contains all this information

#define MAXARGSSIZE	1024
typedef struct {
	char cmdline[MAXARGSSIZE]; // Name of process and its args as in the
				// /proc/<pid>/cmdline file
	struct stat statbuf;	// stat structure for the process entry
				// /proc/<pid>
	pid_t pid;		// pid of the process
} VCSAgProcStatusStruct;

#endif /* !WINNT && LINUX */

#endif /* !WINNT */

//
// Function pointer definitions for the different entry points available
//
typedef unsigned int (*action_fptr) (const char *res_name,
	const char *action_token, void **attr_val, char **action_args,
	char *action_output);

typedef void (*attr_changed_fptr) (const char *res_name, 
	const char *changed_res_name, const char *changed_attr_name,
	void **new_val); 

typedef unsigned int (*clean_fptr) (const char *res_name, VCSAgWhyClean reason,
	void **attr_val);

typedef void (*close_fptr) (const char *res_name, void **attr_val);

typedef unsigned int (*info_fptr) (const char *res_name,
	VCSAgResInfoOp resinfo_op, void **attr_val, char **info_output,
	char ***opt_update_args, char ***opt_add_args);

typedef VCSAgResState (*monitor_fptr) (const char *res_name, void **attr_val, 
	int *conf_level);

typedef unsigned int (*offline_fptr) (const char *res_name, void **attr_val); 

typedef unsigned int (*online_fptr) (const char *res_name, void **attr_val); 

typedef void (*open_fptr) (const char *res_name, void **attr_val); 

typedef void (*shutdown_fptr) ();

typedef int (*amf_init_fptr) ();

typedef int (*amf_getnotification_fptr) (char *resource_id);

typedef unsigned int (*amf_register_fptr) (const char *res_name,
	VCSAgAMFMonitorSwitch mswitch, VCSAgAMFResState rstate,
	void **attr_val, char *resource_id);

//
// Changes for L3+
//
enum VCSAgEncodingType {
	VCSAgUTF8,
	VCSAgUCS2,
	VCSAgStr
};

#define STMTS(s)	do{s}while(0)

#define VCSAG_LOG_INIT(func_name)		\
	const char *_who__ = func_name;	
#define VCSAG_LOG_NULL_INIT()			\
	const char *_who__ = NULL;

#define VCSAG_LOG_ARGS		_who__, __FILE__, __LINE__

#define VCSAgMaxEPOutput 2048 // Script EPs : max number of chars read from the
	                      // script's stdout/err including terminating NULL
	                      // C++ EPs : Size of the output buffer passed to
	                      // the entry point

// Size of buffer to store the resource-id returned by the amf entry points.
// Restricted by VCSAgMaxEPOutput because script EPs return the resource-id
// by printing it to their stdout.
#define VCSAgMaxAMFResIdSize    VCSAgMaxEPOutput

#define VCSAgMaxAMFNumRes 1024 // Current limit on the number of resources
                               // that amf can handle *well*


//
// Depending on whether UNICODE is defined or not,
// the log macros will be mapped to appropriate API calls
//
#ifdef	UNICODE

#ifdef	WINNT

#define VCSAG_LOG_MSG			VCSAgLogMsg2
#define VCSAG_LOGDBG_MSG		VCSAgLogDbgMsg2
#define VCSAG_CONSOLE_LOG_MSG		VCSAgLogConsoleMsg2
#define VCSAG_RES_LOG_MSG		VCSAgResLogMsg2

#else	/* UNICODE && !WINNT */

#define VCSAG_LOG_MSG			VCSAgLogMsg1
#define VCSAG_LOGDBG_MSG		VCSAgLogDbgMsg1
#define VCSAG_CONSOLE_LOG_MSG		VCSAgLogConsoleMsg1
#define VCSAG_RES_LOG_MSG		VCSAgResLogMsg1

#endif	/* UNICODE && !WINNT */

#else	/* !UNICODE */

#define VCSAG_LOG_MSG			VCSAgLogMsg1
#define VCSAG_LOGDBG_MSG		VCSAgLogDbgMsg1
#define VCSAG_CONSOLE_LOG_MSG		VCSAgLogConsoleMsg1
#define VCSAG_RES_LOG_MSG		VCSAgResLogMsg1

#endif	/* !UNICODE */

#ifdef	WINNT

//
// Memory Allocation APIs
//
#define VCSAgNew(type, size)	(type *)VCSAgAllocMem(sizeof(type) * (size))
#define VCSAgDelete(Ptr)		VCSAgDeleteMem((void *)(Ptr))

#endif	/* WINNT */

//
// Flags for agent logging
//
#define VCS_DEFAULT_FLAGS	0x00000000
#define VCS_NO_ENDL		0x00000001
#define VCS_NO_TIME		0x00000002
#define VCS_NO_SEV		0x00000004
#define VCS_NO_UMI		0x00000008
#define VCS_NO_CONSOLE		0x00000010
#define VCS_DBG			0x00000020
#define VCS_NO_DBG		0x00000040
#define VCS_NO_MNEM		0x00000080
#define VCS_FORCE_CONSOLE	0x00000100
#define VCSAG_FORCE		0x00000200
#define VCS_CLI			(VCS_NO_TIME|VCS_NO_CONSOLE)
#define VCS_OUTPUT		(VCS_CLI|VCS_NO_UMI|VCS_NO_MNEM|VCS_NO_SEV)
#define VCS_NONE		(VCS_OUTPUT|VCS_NO_ENDL)

enum VCSAgAppSev {
	AG_CRITICAL,
	AG_ERROR,
	AG_WARNING,
	AG_NOTICE,
	AG_INFORMATION
};


enum VCSAgDbgSev {
	DBG1,
	DBG2,
	DBG3,
	DBG4,
	DBG5,
	DBG6,
	DBG7,
	DBG8,
	DBG9,
	DBG10,
	DBG11,
	DBG12,
	DBG13,
	DBG14,
	DBG15,
	DBG16,
	DBG17,
	DBG18,
	DBG19,
	DBG20,
	DBG21,
	DBG_SEV_END
};

#define VCS_CRITICAL			AG_CRITICAL, VCSAG_LOG_ARGS	
#define VCS_ERROR			AG_ERROR, VCSAG_LOG_ARGS
#define	VCS_WARNING			AG_WARNING, VCSAG_LOG_ARGS
#define VCS_NOTICE			AG_NOTICE, VCSAG_LOG_ARGS
#define VCS_INFORMATION			AG_INFORMATION, VCSAG_LOG_ARGS

#define VCS_DBG1			DBG1, VCSAG_LOG_ARGS	
#define VCS_DBG2			DBG2, VCSAG_LOG_ARGS
#define VCS_DBG3			DBG3, VCSAG_LOG_ARGS
#define VCS_DBG4			DBG4, VCSAG_LOG_ARGS
#define VCS_DBG5			DBG5, VCSAG_LOG_ARGS
#define VCS_DBG6			DBG6, VCSAG_LOG_ARGS
#define VCS_DBG7			DBG7, VCSAG_LOG_ARGS
#define VCS_DBG8			DBG8, VCSAG_LOG_ARGS
#define VCS_DBG9			DBG9, VCSAG_LOG_ARGS
#define VCS_DBG10			DBG10, VCSAG_LOG_ARGS
#define VCS_DBG11			DBG11, VCSAG_LOG_ARGS
#define VCS_DBG12			DBG12, VCSAG_LOG_ARGS
#define VCS_DBG13			DBG13, VCSAG_LOG_ARGS
#define VCS_DBG14			DBG14, VCSAG_LOG_ARGS
#define VCS_DBG15			DBG15, VCSAG_LOG_ARGS
#define VCS_DBG16			DBG16, VCSAG_LOG_ARGS
#define VCS_DBG17			DBG17, VCSAG_LOG_ARGS
#define VCS_DBG18			DBG18, VCSAG_LOG_ARGS
#define VCS_DBG19			DBG19, VCSAG_LOG_ARGS
#define VCS_DBG20			DBG20, VCSAG_LOG_ARGS
#define VCS_DBG21			DBG21, VCSAG_LOG_ARGS

#ifdef	WINNT 
typedef unsigned long VCSAgErrnoType;
#else	/* !WINNT */
typedef int VCSAgErrnoType;
#endif	/* !WINNT */

#ifdef  __cplusplus
}
#endif	/* __cplusplus */


#endif	/* !_VCSAgApiDefs_h_ */
