/* $Id: agent.C,v 1.1 2006/04/04 23:49:32 ggoel Exp $  */
/* $Copyrights: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $ */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "VCSAgApi.h"

/* Logging :
 * There are 4 logging APIs - interfaced via macros with the agent
 * entry points. These macros are:
 *
 * VCSAG_CONSOLE_LOG_MSG(sev, msgid, flags, fmt, variable_args...)
 * VCSAG_LOG_MSG(sev, msgid, flags, fmt, variable_args...)
 *
 * ***** No msg_id for debug messages *****
 * 
 * VCSAG_LOGDBG_MSG(dbgsev, flags, fmt, variable_args...)
 * VCSAG_RES_LOG_MSG(dbgsev, flags, fmt, variable_args...)
 * 	
 * 	The arguments to the macros are described below:
 * 	sev: Application severity in the range 
 * 		VCS_CRITICAL - VCS_INFORMATION
 * 	msgid: 16 bit integer msg id
 * 	flags: Default flags(0) - will print UMI+NEWLINE. 
 * 		There is a macro for the default flags: VCS_DEFAULT_FLAGS
 * 	fmt: format string with all the symbols unqualified e.g.
 * 		"Unable to delete file(%s). Delete failed with error(%d)"
 *	variable_args: variable number of arguments (upto 6) of type char, char * or integer.
 *	dbgsev: Debug severity in the range VCS_DBG1 - VCS_DBG21
 *	
 *
 * VCS_CRITICAL, VCS_ERROR, VCS_WARNING, VCS_NOTICE, VCS_INFORMATION are the 
 * application sev macros and they should be used in the log macros: 
 * VCSAG_CONSOLE_LOG_MSG and VCSAG_LOG_MSG. 
 *
 * VCS_DBG1, VCS_DBG2, ..., VCS_DBG21 are debug sev macros and they should be used
 * in the macros VCSAG_LOGDBG_MSG and VCSAG_RES_LOG_MSG. 
 */	


/*The attr_changed entry point is called when a resource attribute is modified, and only
 * if that resource is registered with the agent framework for notification. */
extern "C"
void res_attr_changed
(const char *res_name, const char *changed_res_name,
 const char *changed_attr_name, void **new_val)
{
	 int i=0;
    	 char **my_new_val=(char**)new_val;
	/*
     	* For illustration purposes only. 
     	* This sample function just logs a message about the change 
     	* to the agent log file.  
     	*/
	 
	/* If there are any log messages in the function then to initialise the logging
	 * you should use VCSAG_LOG_INIT initially as shown below. 
	 */
 	 
	VCSAG_LOG_INIT("res_attr_changed");

	VCSAG_LOG_MSG(VCS_INFORMATION,1001,VCS_DEFAULT_FLAGS,"Attribute (%s) changed.", changed_attr_name);
	VCSAG_LOG_MSG(VCS_INFORMATION,1002,VCS_DEFAULT_FLAGS,"Begin - new value");

    	while (new_val && my_new_val[i]) {
		/* This msg will be printed for all resources if VCS_DBG4 is enabled
		 * for the resource type. Else it will be logged only for that 
		 * resource that has the dbg level VCS_DBG4 enabled
		 */
		 VCSAG_RES_LOG_MSG(VCS_DBG4, VCS_DEFAULT_FLAGS, "Value of i:(%d)",i);
			
	        i++;
    	}
	VCSAG_LOG_MSG(VCS_INFORMATION,1003,VCS_DEFAULT_FLAGS,"End - new value");
}


/* The clean entry point is called automatically by the agent framework when all ongoing
 * actions associated with a resource must be terminated and the resource must be taken
 * offline, perhaps forcibly. */
extern "C"
unsigned int res_clean
(const char *res_name, VCSAgWhyClean reason, void **attr_val)
{
    	/*
     	* Perform actions to forcibly bring the resource OFFLINE.
     	* If successful, return 0; else return 1.
     	*/
    	return 0;
}

/* The monitor entry point typically contains the code to determine status of the resource. */
extern "C"
VCSAgResState res_monitor
(const char *res_name, void **attr_val, int *conf_level)
{
	/*
	 * This entry point is mandatory. 
	 * Determine if the given res is online (res exists) or offline (res does
	 * not exist).
	 */
	
    	/* Intialize the OUT params. */
    	VCSAgResState state = VCSAgResUnknown;
    	*conf_level=0;

    	/* Determine the state & confidence level of the resource. */
    	return state;
}

/* The offline entry point is called to take a resource offline. */
extern "C"
unsigned int res_offline(const char *res_name, void **attr_val) { 

    	/* 
     	* Code to offline the resource. 
	*
	* 
     	* Return value indicates howlong to wait before calling monitor
     	* to see if the resource is indeed OFFLINE. The return value in
     	* most of the cases will be 0.
     	*/
    	return 0;
}

/* The open entry point is called when the VCS agent starts managing a resource;*/
extern "C"
void res_open(const char *res_name, void **attr_val)
{
    	/*
     	* For illustration purposes only. 
     	* Register for changes to the Foo attribute of this resource.
     	*/
    	VCSAgRegister(res_name, res_name, "restype");

    	/*
     	* Any code to initialize the resource. When the agent comes up,
     	* this function is guaranteed to be called before online, offline
     	* or monitor.
     	*/
}

/* The shutdown entry point is called before the VCS agent shuts down. */
extern "C" void res_shutdown() {}

VCSAGEXPORT void VCSAgStartup ()
{
	VCSAgV40EntryPointStruct ep;

        ep.open         = res_open;
        ep.close        = NULL;		/* expecting script entry point */
        ep.monitor      = res_monitor;
        ep.online       = NULL;		/* expecting script entry point */
        ep.offline      = res_offline;
        ep.clean        = res_clean;
        ep.attr_changed = res_attr_changed;
        ep.shutdown     = res_shutdown;
	ep.action 	= NULL;		
	ep.info 	= NULL;

	VCSAgSetLogCategory (10061);
	VCSAgRegisterEPStruct(V50, &ep);
}
