/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef GABHDR_H
#define GABHDR_H

static char sccsid_gabhdr_h[] = "@(#)comms:$RCSfile: gabhdr.h,v $	$Revision: 4.2.36.1 $";

#if defined(_KERNEL) /* { */

#if defined(__linux__)

#include <linux/string.h>

#elif defined(__hpux)

#include <sys/kern_svcs.h>

#elif defined(__sun)
 
#include <sys/types.h>
#include <sys/sunddi.h>

#endif

#else	/* } _KERNEL { */

#include <string.h>

#endif /* } _KERNEL */


/* Interpretation of registration flags:
 *
 *     GAB_IOFENCE    I/O fencing option.  Instead of killing a node
 *		      the effected application will receive an IOFENCE
 *		      message.  If the port is not closed within
 *		      the configurable I/O fence timeout period, the 
 *		      node will be killed (halted).
 *
 * GAB_NONBLOCK for non-blocking IO.  When enabled, gab_send() call will
 * return EAGAIN during flow control and gab_receive() will return EAGAIN
 * when no messages are available.  Otherwise these calls will block.
 */

#define GAB_IOFENCE	0x100	/* port eject message			*/
#define GAB_NONBLOCK    0x200	/* non blocking user level i/o 		*/
#define GAB_PROCDEATH	0x400	/* user process death notification 	*/
#define GAB_JNOTIFY	0x800	/* jeopardy notification 		*/

/* macros used for comparing seq and gen numbers
 */
#define GAB_NULL	0
#define GAB_LT(x, y)	((!(x) && (y)) || \
			((((x) - (y)) & 0x80000000) && (y)))
#define GAB_GT(x, y)	GAB_LT((y),(x))
#define GAB_EQ(x, y)	((x) == (y))
#define GAB_INC(x)	((x)+1 ? (x)+1 : 1)
#define GAB_DEC(x)	((x)-1 ? (x)-1 : -1)

#define  GAB_ISSETBIT(map, b)      \
	(map[(b)/(8*sizeof(map[0]))] & (1 << ((b) & (8*sizeof(map[0]) - 1))))
#define  GAB_SETBIT(map, b)      \
	(map[(b)/(8*sizeof(map[0]))] |= (1 << ((b) & (8*sizeof(map[0]) - 1))))

/* type and macros for membership masks
 */
#define GAB_MBR_NMBR 4

typedef struct gab_mbr_t {
	unsigned int mbr[GAB_MBR_NMBR];
} gab_mbr_t;

#define GAB_MINID	0
#define GAB_ISSET(set, nid) ((set).mbr[((unsigned int)(nid))>>5] &	\
			      (1<<(((unsigned int)(nid))&31)))
#define GAB_SET(set, nid)   ((set).mbr[((unsigned int)(nid))>>5] |=	\
			      (1<<(((unsigned int)(nid))&31)))
#define GAB_CLR(set, nid)   ((set).mbr[((unsigned int)(nid))>>5] &=	\
			     ~(1<<(((unsigned int)(nid))&31)))

#ifdef _KERNEL

#if defined(__sun) || defined(__hpux) || defined(_AIX)

#define GAB_ZERO(set)   bzero((char *)&(set), sizeof(gab_mbr_t))

#elif defined(__linux__)

#define GAB_ZERO(set)   memset((char *)&(set), 0, sizeof(gab_mbr_t))

#elif defined(WIN32)

#define GAB_ZERO(set)   RtlZeroMemory((char *)&(set), sizeof(gab_mbr_t))

#endif

#else /* !_KERNEL */

#define GAB_ZERO(set)	(void)memset((char *)&(set), '\0', sizeof(gab_mbr_t))

#endif /* _KERNEL */

#define GAB_INTERSECT(dst,src1,src2)				\
		((dst).mbr[0] = (src1).mbr[0] & (src2).mbr[0],	\
		 (dst).mbr[1] = (src1).mbr[1] & (src2).mbr[1], 	\
		 (dst).mbr[2] = (src1).mbr[2] & (src2).mbr[2],	\
		 (dst).mbr[3] = (src1).mbr[3] & (src2).mbr[3])
#define GAB_UNION(dst,src1,src2)				\
		((dst).mbr[0] = (src1).mbr[0] | (src2).mbr[0],	\
		 (dst).mbr[1] = (src1).mbr[1] | (src2).mbr[1], 	\
		 (dst).mbr[2] = (src1).mbr[2] | (src2).mbr[2],	\
		 (dst).mbr[3] = (src1).mbr[3] | (src2).mbr[3])
#define GAB_NOT(dst,src1)					\
		((dst).mbr[0] = ~(src1).mbr[0],			\
		 (dst).mbr[1] = ~(src1).mbr[1],			\
		 (dst).mbr[2] = ~(src1).mbr[2],			\
		 (dst).mbr[3] = ~(src1).mbr[3])
#define GAB_EQUAL(set1,set2)					\
		(((set1).mbr[0] == (set2).mbr[0]) &&		\
		 ((set1).mbr[1] == (set2).mbr[1]) &&		\
		 ((set1).mbr[2] == (set2).mbr[2]) &&		\
		 ((set1).mbr[3] == (set2).mbr[3]))
#define GAB_EMPTY(set)						\
		(((set).mbr[0] == 0) &&				\
		 ((set).mbr[1] == 0) &&				\
		 ((set).mbr[2] == 0) &&				\
		 ((set).mbr[3] == 0))
#define GAB_FFS(dst,src)					\
	(GAB_ZERO(dst),						\
	((src).mbr[0] 						\
	 ?	((dst).mbr[0] = (src).mbr[0] &			\
			((src).mbr[0] ^ ((src).mbr[0] - 1)))	\
	 : ((src).mbr[1]					\
	    ? ((dst).mbr[1] = (src).mbr[1] &			\
			((src).mbr[1] ^ ((src).mbr[1] - 1)))	\
	    :  ((src).mbr[2]					\
		? ((dst).mbr[2] = (src).mbr[2] &		\
			((src).mbr[2] ^ ((src).mbr[2] - 1)))	\
		: ((src).mbr[3]					\
		   ? ((dst).mbr[3] = (src).mbr[3] &		\
	  		((src).mbr[3] ^ ((src).mbr[3] - 1)))	\
		   : 0)))))

#define MBR_STR         "    [%x:%x:%x:%x]"
#define MBR_AR(a)       (a).mbr[0], (a).mbr[1], (a).mbr[2], (a).mbr[3]

/* message types
 */
#define GAB_MSG_BROADCAST	0x01	/* user broadcast data msg	*/
#define GAB_MSG_DIRECTED	0x02	/* user directed data msg	*/
#define GAB_MSG_RECONFIG	0x0a	/* new group membership msg	*/
#define GAB_MSG_IOFENCE		0x0b	/* instructs app to fence&close */
#define GAB_MSG_UNUSED		0x0c	/* was: peer process died msg	*/


/* message flags, given on RECONFIG msg types
 */
#define GAB_FIRST		0x01	/* first membership on this prt */
#define GAB_MORE		0x02	/* more messages avail to read  */
#define GAB_DUPREQ		0x04	/* deplicated seqence request   */

/* message header structure to be prepended to every message (48 bytes)
 */
typedef struct {
	char		gh_port;	/* 1 gab port id		*/
	unsigned char	gh_src;		/* 1 source nid			*/
	unsigned char	gh_type;	/* 1 message type data/reconfig */
	unsigned char	gh_flags;	/* 1 flags for properties of msg*/
	unsigned int	gh_gen;		/* 4 connection generation numbr*/
	unsigned int	gh_seq;		/* 4 broadcast sequence number  */
	unsigned int	gh_size;	/* 4 payload size		*/
	gab_mbr_t	gh_dst;		/*16 destination mask		*/
	unsigned char	gh_filler[16];	/*16 reserved for future use	*/
} gab_msg_hdr_t;

/* macro to get jeopardy and visible membership from RECONFIG message
 * returns gab_mbr_t
 */
#define GAB_GET_JEOVIS(jv, msg)	\
	{				\
		GAB_ZERO((jv));		\
		GAB_UNION((jv), (jv), *((gab_mbr_t *)((msg)+1)));	\
	}

#endif /* GABHDR_H */
