/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef GABUSER_H
#define GABUSER_H

#ifdef __cplusplus
extern "C" {
#endif

static char sccsid_gabuser_h[] = "@(#)comms:$RCSfile: gabuser.h,v $	$Revision: 4.5 $";

#if defined (_WIN64 ) || defined(WINNT) || defined(NTGAB)
#define plat_handle HANDLE
#define plat_badfd NULL
#else
#define plat_handle int
#define plat_badfd -1
#endif

#ifdef LINUX
#include <sys/unistd.h>
#include <stdlib.h>
#endif

#include <sys/gabioc.h>
#include <gabhdr.h>
#include <sys/gaberrno.h>

#ifdef NTGAB
/*
 * Definition for dev_t on NT
 */
typedef INT_PTR dev_t;
#include "gabntusr.h"

#else /* NTGAB */

#include <sys/types.h>

#endif

#define GAB_ENABLE_LOGGING  1
#define GAB_DISABLE_LOGGING 0
#define GAB_DISK_BUSY_RETRIES 10
#define GAB_DISK_BUSY_SLEEP   1    /* sleep 1s between tries */

#define MAXDNAME 100

/*
 * ToDo find the compiler options (had, libgab)
*/
#ifdef _WIN64
#pragma pack(1)
#endif

typedef struct gab_udi_ {
   char      udi_name[MAXDNAME];
   int       udi_start;
   gab_mbr_t udi_active;
   int       udi_flags;
   union {
     dev_t   dev;
     char    fill[8];
   } udi_udev;
#define udi_dev  udi_udev.dev
   unsigned int udi_major;
   unsigned int udi_minor;
   unsigned int udi_portid;
} gab_udi_t;

#ifdef _WIN64
#pragma pack()
#endif
/* 
 * Macros for exporting, importing & calling convention. 
 * Meaningful in NT only.
 */
#define GABEXPORT __declspec(dllexport)
#define GABIMPORT __declspec(dllimport)


/*
 * Specify the linkage for the API functions as export or import depending
 * on whether it is included from the framework or from the agent developer's
 * code.
 */
#ifdef NTGAB
#define GABLINKAGE GABEXPORT
#else
#define GABLINKAGE
#endif


GABLINKAGE int
gab_register(int version,
	     int portid,
	     char *supportlist,
	     int flags,
	     int timeout,
             int signal,
             void (*func)(int)
             );

GABLINKAGE int
gab_pregister(int version,
	      int portid,
	      char *supportlist,
	      int flags,
	      int timeout,
	      int *fd
	      );

GABLINKAGE void
gab_unregister(int portid);

GABLINKAGE void
gab_heartbeat(int portid);

GABLINKAGE void *
gab_allocmsg(int size);

GABLINKAGE int
gab_send(int portid, void *hdr);

GABLINKAGE int
gab_receive(int portid, void *hdr);

GABLINKAGE void
gab_freemsg(void *hdr);

GABLINKAGE unsigned char
gab_maxid();

GABLINKAGE int
gab_diskmonitor(plat_handle fd, dev_t dev, int start, char *devname);

GABLINKAGE int
gab_hbdiskmonitor(plat_handle fd, dev_t *dev, int *start, gab_mbr_t *gab_mbr, 
			int *flags, char *devname);

GABLINKAGE int
gab_getconfig(gab_config_t *config);
 
GABLINKAGE char *
gab_perror(char *cmd_str, int err, int *msgid);

GABLINKAGE int
gab_panic(int portid, gab_panic_t *);

GABLINKAGE void
gab_set_progname(char *);

GABLINKAGE char *
gab_get_progname(void);

GABLINKAGE int
gab_error_reporting(int);

GABLINKAGE int
gab_getdisk(plat_handle, gab_udi_t *);

GABLINKAGE int
gab_get_stable_timeout(void);

GABLINKAGE int
gab_disk_ioctl(plat_handle fd, gab_disk_info_t *info);

#ifdef __cplusplus
}
#endif

#endif /* GABUSER_H */
