/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef GABERRNO_H
#define GABERRNO_H

#ifdef __cplusplus
extern "C" {
#endif

static char sccsid_gaberrno_h[] = "@(#)comms:$RCSfile: gaberrno.h,v $	$Revision: 4.2 $";

#ifdef NTGAB	/* { */

#include "gabnterrno.h"

#else  /* } NTGAB { */

/* On AIX 4.3, errno is char (1 byte).  So, these
 * errno values should be less than 255
 */
#ifdef _AIX /* { */

#define ECLIENTSEXIST          156
#define ELLTNOTCONF            157
#define EBADVERSION            158
#define EBADSIMMODE            159
#define ENOSIMPOINT            160
#define EGABNOTCONF            161
#define EBADPORT               162
#define ESEND                  163

#else /* } _AIX { */

#define ECLIENTSEXIST          256
#define ELLTNOTCONF            257
#define EBADVERSION            258
#define EBADSIMMODE            259
#define ENOSIMPOINT            260
#define EGABNOTCONF            261
#define EBADPORT               262
#define ESEND                  263

#endif /* } */

#ifdef _KERNEL /* { */

#ifdef __linux__

/*
 * linux uses negative errno in kernel
 */
#define PLAT_KERR_PREFIX   -

#else /* Solaris/HP/AIX */

/*
 * kernel error number prefix (+/-)
 */
#define PLAT_KERR_PREFIX

#endif	/* __linux__ */

/*
 * Kernel error code mapping.
 */
#define GAB_KE_NOENT           (PLAT_KERR_PREFIX ENOENT)
#define GAB_KE_INTR            (PLAT_KERR_PREFIX EINTR)
#define GAB_KE_IO              (PLAT_KERR_PREFIX EIO)
#define GAB_KE_NXIO            (PLAT_KERR_PREFIX ENXIO)
#define GAB_KE_AGAIN           (PLAT_KERR_PREFIX EAGAIN)
#define GAB_KE_NOMEM           (PLAT_KERR_PREFIX ENOMEM)
#define GAB_KE_ACCES           (PLAT_KERR_PREFIX EACCES)
#define GAB_KE_FAULT           (PLAT_KERR_PREFIX EFAULT)
#define GAB_KE_BUSY            (PLAT_KERR_PREFIX EBUSY)
#define GAB_KE_EXIST           (PLAT_KERR_PREFIX EEXIST)
#define GAB_KE_NODEV           (PLAT_KERR_PREFIX ENODEV)
#define GAB_KE_INVAL           (PLAT_KERR_PREFIX EINVAL)
#define GAB_KE_NOSR            (PLAT_KERR_PREFIX ENOSR)
#define GAB_KE_PROTO           (PLAT_KERR_PREFIX EPROTO)
#define GAB_KE_CLIENTSEXIST    (PLAT_KERR_PREFIX ECLIENTSEXIST)
#define GAB_KE_LLTNOTCONF      (PLAT_KERR_PREFIX ELLTNOTCONF)
#define GAB_KE_BADVERSION      (PLAT_KERR_PREFIX EBADVERSION)
#define GAB_KE_BADSIMMODE      (PLAT_KERR_PREFIX EBADSIMMODE)
#define GAB_KE_NOSIMPOINT      (PLAT_KERR_PREFIX ENOSIMPOINT)
#define GAB_KE_GABNOTCONF      (PLAT_KERR_PREFIX EGABNOTCONF)
#define GAB_KE_BADPORT         (PLAT_KERR_PREFIX EBADPORT)
#define GAB_KE_SEND            (PLAT_KERR_PREFIX ESEND)
#define GAB_KE_NOTCONN         (PLAT_KERR_PREFIX ENOTCONN)
#define GAB_KE_MSGSIZE         (PLAT_KERR_PREFIX EMSGSIZE)

#if defined(__hp) || defined(_AIX) /* EBADR doesn't exist on HP/AIX */

#define GAB_KE_BADR            (PLAT_KERR_PREFIX EFAULT)

#else

#define GAB_KE_BADR            (PLAT_KERR_PREFIX EBADR)

#endif  /* __hp/_AIX */

#else /* } UNIX _KERNEL { */

/*
 * gab user error code mapping.
 */
#define GAB_UE_NOENT           ENOENT
#define GAB_UE_INTR            EINTR
#define GAB_UE_IO              EIO
#define GAB_UE_NXIO            ENXIO
#define GAB_UE_AGAIN           EAGAIN
#define GAB_UE_NOMEM           ENOMEM
#define GAB_UE_ACCES           EACCES
#define GAB_UE_FAULT           EFAULT
#define GAB_UE_BUSY            EBUSY
#define GAB_UE_EXIST           EEXIST
#define GAB_UE_NODEV           ENODEV
#define GAB_UE_INVAL           EINVAL
#define GAB_UE_NOSR            ENOSR
#define GAB_UE_PROTO           EPROTO
#define GAB_UE_CLIENTSEXIST    ECLIENTSEXIST
#define GAB_UE_LLTNOTCONF      ELLTNOTCONF
#define GAB_UE_BADVERSION      EBADVERSION
#define GAB_UE_BADSIMMODE      EBADSIMMODE
#define GAB_UE_NOSIMPOINT      ENOSIMPOINT
#define GAB_UE_GABNOTCONF      EGABNOTCONF
#define GAB_UE_BADPORT         EBADPORT
#define GAB_UE_SEND            ESEND
#define GAB_UE_NOTCONN         ENOTCONN
#define GAB_UE_MSGSIZE         EMSGSIZE

#define GAB_NATIVE_ERR         0x80000000

#if defined(__hpux) || defined(_AIX)

#define GAB_UE_BADR            EFAULT

#else

#define GAB_UE_BADR            EBADR

#endif  /* __hp/_AIX */

#endif /*  } UNIX _KERNEL */

#endif /* } NTGAB */

#ifdef __cplusplus
}
#endif

#endif /* GABERRNO_H */
