/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef   GABKRL_H
#define   GABKRL_H

static char sccsid_gabkrl_h[] = "@(#)comms:$RCSfile: gabkrl.h,v $	$Revision: 4.14.2.4 $";

#if defined(__sun)

#include <sys/types.h>
#include <sys/cmn_err.h>
#include <sys/stream.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>
#include <sys/devops.h>

#elif defined(__hpux)

#include <sys/types.h>
#include <sys/cmn_err.h>
#include <sys/stream.h>

#endif

#include <gabhdr.h>
#include <sys/gabmsg.h>
#include <sys/gaberrno.h>
#include <sys/gabvers.h>

/* Message list anchor
 */
typedef struct gab_mlist {
	void	*gml_forw;
	void	*gml_back;
} gab_mlist_t;

/* client callbacks
 */
typedef struct gab_funcs {
	void		(*gf_recv)(void *cookie, void *hdr);
	void		(*gf_backenable)(void *cookie);
	int		(*gf_canput)(void *cookie);
	void 		*gf_cookie;
	void		(*gf_Xunused)(void);
} gab_funcs_t;

typedef char gab_portid_t;

/* private api struct
 */
typedef struct gab_api {
	void		*gabapi_devinfo;
	int		(*gabapi_register)(int version,
					   char portid,
					   char *supportlist,
					   gab_funcs_t *funcs, 
					   int flags);
	void		(*gabapi_unregister)(char portid);
	void		*(*gabapi_allocmsg)(int size);
	int		(*gabapi_send)(char portid, void *hdr);
	void		(*gabapi_freemsg)(void *hdr);
	int		(*gabapi_canput)(char portid);
	void		(*gabapi_backenable)(char portid);
	unsigned char	(*gabapi_maxid)(void);

	/* glm api -- for binary compatibility only
	 */
	void		(*Xglmapi_restart)(void);
	int		(*Xglmapi_initlock)(void);
	int		(*Xglmapi_deinitlock)(void);

	/* glmapi_lock is overloaded with all lock/try/async/recursive ops
	 */
	int		(*Xglmapi_lock)(void);
	int		(*Xglmapi_ready)(void);
	int		(*Xglmapi_release)(void);
	int		(*Xglmapi_reload)(void);
	void		(*Xglmapi_freedata)(void);

	/* Message list entry points
	 */
	void		(*gabapi_mlist_init)(gab_mlist_t *list);
	void		(*gabapi_mlist_deinit)(gab_mlist_t *list);
	void		(*gabapi_mlist_add)(gab_mlist_t *list,
						void	*msg,
						int	front);
	void		*(*gabapi_mlist_rem)(gab_mlist_t *list, int front);
	void		(*gabapi_mlist_move)(gab_mlist_t *src,
						gab_mlist_t *dst);
	int             (*gabapi_mlist_empty)(gab_mlist_t *list);

	/* BL5 interfaces below here
         */
        int             (*gabapi_register_compat)(int version,
                                           char portid,
                                           char *supportlist,
                                           gab_funcs_t *funcs,
                                           int flags);
        void            (*Xglmapi_async_cancel)(void);
	/* Ghost locks
	 */
	int		(*Xglmapi_ghost_lock)(void);
	int		(*Xglmapi_ghost_unlock)(void);

	void		(*gabapi_hold)(void);
	void		(*gabapi_release)(void);
} gab_api_t;

#if defined(__sun)




extern struct dev_ops *mod_hold_dev_by_devi(dev_info_t *devi);
extern void   mod_rele_dev_by_devi(dev_info_t *devi);
 
#define GAB_API_INIT(api) { 						\
	dev_info_t *dp; 						\
	(api) = NULL;		     					\
        if (ddi_pathname_to_dev_t("/pseudo/gab@0:gab_0") == (dev_t)-1) {\
		;							\
	} else if ((dp = ddi_find_devinfo("gab", -1, 0)) == NULL) {	\
                cmn_err(CE_CONT, "GAB ERROR V-15-1-20135 GAB not initialized\n");		\
        } else if (mod_hold_dev_by_devi(dp) == NULL) {			\
                cmn_err(CE_CONT, "GAB ERROR V-15-1-20136 Can not hold GAB\n");		\
        } else {     							\
		(api) = (struct gab_api *)ddi_get_driver_private(dp);	\
		if (api)						\
			((api)->gabapi_hold)();				\
	}     								\
}

#define GAB_API_DEINIT(api) 						\
	((api)->gabapi_release)();					\
	(mod_rele_dev_by_devi((dev_info_t *)(api)->gabapi_devinfo),	\
	(api) = NULL)

#elif defined(__linux__)

#include <linux/module.h>
#include <linux/version.h>
#include <sys/gab_imc.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18))
extern void *gab_imc_import(char *, char *);
#define GAB_API_INIT(api) {						\
	(api) = (gab_api_t *)gab_imc_import("gabapi","gab");		\
	if ((api) == NULL) {	\
		printk(KERN_ERR "GAB ERROR V-15-1-20137 GAB is not loaded\n");\
	}								\
}
#else
#define GAB_API_INIT(api) {						\
	(api) = inter_module_get_request("gabapi", "gab");		\
	if ((api) == NULL) {	\
		printk(KERN_ERR "GAB ERROR V-15-1-20137 GAB is not loaded\n");	\
	}								\
}
#endif


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18))
extern void gab_imc_deport(char *);
#define GAB_API_DEINIT(api)						\
	((api) = NULL, gab_imc_deport("gabapi"))
#else
#define GAB_API_DEINIT(api)						\
	((api) = NULL, inter_module_put("gabapi"))
#endif


#elif defined(UNIXWARE)




#define GAB_API_INIT(api) 	(api) = &gab_api_local

#define GAB_API_DEINIT(api) 	(api) = NULL



#elif defined(__hpux)
#ifndef HPUX1131 /* not HPUX1131 */
/*
 *  Backfill for missing HP declaration.
 */
extern int mod_obj_getsymp(const char *, const char *, int, Elf64_Sym *);
#define GAB_API_INIT(api)  {                                        \
        Elf64_Sym sym;                                              \
        if (mod_obj_getsymp((const char *)"gab_api_local", NULL, 0, \
                &sym) == 0) {                                  \
                (api) = (gab_api_t *) sym.st_value;            \
        }                                                      \
}
#else /* HPUX1131 */
#define GAB_API_INIT(api)  {                                       \
        uint64_t sym;                                              \
        (void)getsymaddr((const char *)"gab_api_local", "gab", 0,  \
                &sym);                                             \
        if (sym) {                                                 \
                (api) = (gab_api_t *) sym;                         \
        }                                                          \
}
#endif /* HPUX1131 */

#define GAB_API_DEINIT(api)     (api) = NULL

#elif defined(_AIX)

#ifndef NO_VEKI

typedef struct gab_veki_api {
	ki_t gab_ki;
	struct gab_api *gab_api;
} gab_veki_api_t;

#define GABAPI_NAME	"veritas.gab"
#define GABAPI_VERSION	"1"

#define GAB_API_INIT(api)
#define GAB_API_DEINIT(api)

#else 

extern struct gab_api gab_api_local;
#define GAB_API_INIT(api) 	(api) = &gab_api_local
#define GAB_API_DEINIT(api) 	(api) = NULL

#endif /* NO_VEKI */

#elif defined(NTGAB)

#define GAB_API_INIT(api)   gabInitApi(&api)
#define GAB_API_DEINIT(api) gabFreeApi(api)

#endif /* PLATFORM */




#define GAB_API_REGISTER(api,version,portid,supportlist,funcs,flags)	\
	(((api) && (api)->gabapi_register) ?				\
	 ((api)->gabapi_register)((version),(portid), (supportlist),	\
				  (funcs),(flags))			\
	 : GAB_KE_BADR)

#define GAB_API_UNREGISTER(api, portid)					\
	(((api) && (api)->gabapi_unregister) ?				\
		((api)->gabapi_unregister)(portid) 			\
	: (void)0)

#define GAB_API_ALLOCMSG(api, size)					\
	(((api) && (api)->gabapi_allocmsg) ?				\
	 ((api)->gabapi_allocmsg)((int)(size))				\
	 : NULL)

#define GAB_API_SEND(api, portid, hdr)					\
	(((api) && (api)->gabapi_send) ?				\
	 ((api)->gabapi_send)((portid), (void *)(hdr))			\
	 : GAB_KE_BADR)

#define GAB_API_FREEMSG(api, hdr)					\
	(((api) && (api)->gabapi_freemsg) ?				\
		((api)->gabapi_freemsg)((void *)(hdr))	 		\
	: (void)0)

#define GAB_API_CANPUT(api, portid)					\
	(((api) && (api)->gabapi_canput) ?				\
	 ((api)->gabapi_canput)(portid)					\
	 : 0)

#define GAB_API_BACKENABLE(api, portid)					\
	(((api) && (api)->gabapi_backenable) ?				\
		((api)->gabapi_backenable)(portid)			\
	: (void)0)

#define GAB_API_MAXID(api)						\
	(((api) && (api)->gabapi_maxid) ?				\
	 ((api)->gabapi_maxid)()					\
	 : 0)


/* Message lists
 */
#define GAB_API_MLIST_INIT(api, list)					\
	(((api) && (api)->gabapi_mlist_init) ?				\
		((api)->gabapi_mlist_init)(list)			\
	: (void)0)

#define GAB_API_MLIST_DEINIT(api, list)					\
	(((api) && (api)->gabapi_mlist_deinit) ?			\
		((api)->gabapi_mlist_deinit)(list) 			\
	: (void)0)

#define GAB_API_MLIST_ADD(api, list, msg, front)			\
	(((api) && (api)->gabapi_mlist_add) ?				\
		((api)->gabapi_mlist_add)((list), (msg), (front)) 	\
	: (void)0)

#define GAB_API_MLIST_REM(api, list)					\
	(((api) && (api)->gabapi_mlist_rem) ?				\
	((api)->gabapi_mlist_rem)((list), 1)				\
	: NULL)

#define GAB_API_MLIST_REM_REAR(api, list)				\
	(((api) && (api)->gabapi_mlist_rem) ?				\
	((api)->gabapi_mlist_rem)((list), 0)				\
	: NULL)

#define GAB_API_MLIST_MOVE(api, src, dst)				\
	(((api) && (api)->gabapi_mlist_move) ?				\
		((api)->gabapi_mlist_move)((src), (dst))		\
	: (void)0)

#define GAB_API_MLIST_EMPTY(api, list)                                  \
        (((api) && (api)->gabapi_mlist_empty) ?                         \
        ((api)->gabapi_mlist_empty)(list)                               \
        : 1)

#endif /* GABKRL_H */
