/*
 * $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $
 */

#ifndef   GABMSG_H
#define   GABMSG_H

static char sccsid_gabmsg_h[] = "@(#)comms:$RCSfile: gabmsg.h,v $	$Revision: 4.10.6.2 $";

#if defined(__sun)

#include <sys/byteorder.h>

#endif

#if defined(__sun) || defined(__hpux) || defined(_AIX)

#include <sys/stream.h> 

#endif  

#include <sys/lltapi.h>

/****************************************************************************/
/* GAB message data structures and macros                                   */
/*****************************************************************************/

/*
 * PRIVATE
 */
typedef union {
	unsigned int    ptr_val;
	char            filler[8];
} mblk_ptr_t;

typedef struct gab_msg_t {
        /* private header (48 bytes)
	 */
        mblk_ptr_t      gm_umblk;       /* 8 back link to mblk_t struct */
#define gm_mblk         gm_umblk.ptr_val
        char            gm_filler0[8];  /* 8 reserved for future use    */
	unsigned char	gm_version;	/* 1 on the wire version number */
	unsigned char	gm_src;		/* 1 message source nid		*/
	unsigned char	gm_state;	/* 1 message state also type ext*/
	unsigned char	gm_iflags;	/* 1 internal flags eg BINENDIAN*/
	unsigned int	gm_src_seq;	/* 4 broadcastor source sequence*/
        unsigned int    gm_src_gen;     /* 4 original generation        */
	unsigned int	gm_cnt;		/* 4 seq request & delivery ordr*/
	gab_mbr_t	gm_dst;		/*16 message destination mask	*/

        /* user header (48 bytes)
	 */
	char		gm_portid;	/* 1 llt port id		*/
	unsigned char	gm_bsrc;	/* 1 broadcast source nid	*/
	unsigned char	gm_type;	/* 1 message type data/reconfig */
	unsigned char	gm_flags;	/* 1 flags for properties of msg*/
	unsigned int	gm_gen;		/* 4 connection generation nbr  */
	unsigned int	gm_seq;		/* 4 broadcast sequence number  */
	unsigned int	gm_size;	/* 4 payload size		*/
	gab_mbr_t	gm_bdst;	/*16 broadcast/uesr destination */
        unsigned char   gm_filler2[16]; /*16 reserved for future use    */
} gab_msg_t;

#if defined(_LP64) || defined(__LP64__) || defined (_WIN64) \
		|| defined(__powerpc64__) /* { */

#define GAB_MBLKASSIGN(dst,src)                                         \
        (*((int *)&(dst))=*((int *)&(src)),                             \
         *(((int *)&(dst))+1)=*(((int *)&(src))+1))

#define GAB_MBLKNULL(dst)                                               \
        (*((int *)&(dst))=0,                                            \
         *(((int *)&(dst))+1)=0)

# define GAB_MBLKREF(src) \
         gab_align_mblk_ptr((unsigned int*)(void*)&(src))

# undef gab_inl
# if  defined(__hpux) || defined(_AIX) || defined (_WIN64)
#   pragma INLINE  gab_align_mblk_ptr
#   define gab_inl
# else
#   define gab_inl inline
# endif

  static gab_inl mblk_t* gab_align_mblk_ptr(unsigned int* p) {
    union {
        mblk_t         *mp;
        unsigned int    a[2];
    } un;

    un.a[0] = p[0];
    un.a[1] = p[1];
    return un.mp;
  }

#else    /* } __LP64__ { */

#define GAB_COERCE_TO_LVALUE(x)         (*(void**)&(x))
#define GAB_MBLKASSIGN(dst,src)         \
        (GAB_COERCE_TO_LVALUE(dst) = GAB_COERCE_TO_LVALUE(src))
#define GAB_MBLKNULL(dst)               ((dst) = NULL)

#if defined (_WIN64)
#define GAB_MBLKREF(src)                ((mblk_t*)ULongToPtr((src)))
#else
#define GAB_MBLKREF(src)                ((mblk_t*)(src))
#endif

#endif   /* } __LP64__ */

/*
 * macros for acccessing mblk pointer field
 */
#ifndef __linux__

#define MBLK_WPTR(mp)        ((mp)->b_wptr)
#define MBLK_RPTR(mp)        ((mp)->b_rptr)
#define MBLK_NEXT(mp)        ((mp)->b_next)
#define MBLK_PREV(mp)        ((mp)->b_prev)
#define MBLK_CONT(mp)        ((mp)->b_cont)
/*
 * not defined yet, will add once b_flag is added to lltmblk
 * #define MBLK_FLAG(mp)     ((mp)->b_flag)
 */
#define MBLK_PUT(mp, size)   ((mp)->b_wptr += size)  /* put data in mblk */
#define MBLK_PUT_NORET       MBLK_PUT
#define MBLK_PULL(mp, size)  ((mp)->b_rptr += size)  /* return new rptr */

#else

#define MBLK_RPTR(mp)        ((mp)->data)
#define MBLK_WPTR(mp)        ((mp)->tail)
#define MBLK_NEXT(mp)        ((mp)->next)
#define MBLK_PREV(mp)        ((mp)->prev)
#define MBLK_CONT(mp)        ((mp)->list)

/*
 * not defined yet, need to reuse another field in skbuff
 * #define MBLK_FLAG(mp)     ((mp)->b_flag)
 */
#define MBLK_PUT(mp, size)         (skb_put((mp),(size))+size)
#define MBLK_PUT_NORET(mp, size)   (void)(skb_put((mp),(size)))
#define MBLK_PULL(mp, size)        skb_pull((mp),(size))

#endif

/*
 *  GAB messages have *FOUR* addresses associated with them:
 *  1.  The address of all the data and pointed to by mp->b_rptr
 *  2.  The address of the user portion of the GAB header
 *  3.  The address of the user data (follows all GAB header data)
 *  4.  The address of the mblk_t structure for managing the message
 *
 *  The following macros refer to these as, respectively,
 *  "GHDR", "HDR", "PAYLOAD" and "MBLK".  The "cannonical" address
 *  is the "HDR" form, so we provide 6 macros to convert to and from
 *  that form.
 */
#define GAB_HDR2GHDR(p)     \
	((gab_msg_t*)(void*)((char*)(p) + sizeof(gab_msg_hdr_t)-sizeof(gab_msg_t)))

#define GAB_HDR2PAYLOAD(p)  ((char*)((p)+1UL))
#define GAB_HDR2MBLK(p)     GAB_MBLKREF((GAB_HDR2GHDR(p)->gm_mblk))

#define GAB_PAYLOAD2HDR(p)  \
	((gab_msg_hdr_t*)(void*)((char*)(p) - sizeof(gab_msg_hdr_t)))

#define GAB_GHDR2HDR(p)     \
	((gab_msg_hdr_t*)(void*)(((char*)((p)+1UL)) - sizeof(gab_msg_hdr_t)))

#define GAB_MBLK2HDR(p)     GAB_GHDR2HDR(((gab_msg_t *)(void*)(MBLK_RPTR(p))))
#define GAB_MSGSIZE(size1)  (sizeof(gab_msg_t)+(size1)-sizeof(gab_msg_hdr_t))
#define GAB_EOMSG(msg1)     (((char *)(msg1))+GAB_MSGSIZE((msg1)->gm_size))

/*
 *  Deprecated forms:
 */
#define GAB_PAYLOAD(msg1)   GAB_HDR2PAYLOAD(msg1)
#define GAB_MSGHDR(msg1)    GAB_GHDR2HDR(msg1)
#define GAB_HDRMSG(hdr1)    GAB_HDR2GHDR(msg1)
#define GAB_MBLKMSG(mp)     GAB_MBLK2HDR(mp)
#define GAB_HDR2UHDR(p)     GAB_GHDR2HDR(p)
#define GAB_UHDR2HDR(p)     GAB_HDR2GHDR(p)

#endif /* GABMSG_H */
